# THIS FILE IS GENERATED FROM ORANGE-WIDGET-BASE SETUP.PY
short_version = '4.12.0'
version = '4.12.0'
full_version = '4.12.0'
git_revision = '0598a2d03407002f98984ebbb2e841797d79c23a'
release = True

if not release:
    version = full_version
    short_version += ".dev"
